<?php
/**
* @package    Playthru
* @copyright  Copyright (c)2011-2013 Davide Tampellini
* @license    GNU General Public License version 3, or later
*/
defined('_JEXEC') or die;

class plgCaptchaPlaythru extends JPlugin
{
	const AYAH_WEB_SERVICE_HOST = "ws.areyouahuman.com";

	protected $ayah_publisher_key;
	protected $ayah_scoring_key;

	protected $session_secret;

	public function __construct($subject, $config)
	{
		parent::__construct($subject, $config);

		$this->session_secret 	  = JRequest::getVar('session_secret');
		$this->ayah_scoring_key   = $this->params->get('scoring_key', '');
		$this->ayah_publisher_key = $this->params->get('publisher_key', '');

		$this->loadLanguage();
	}

	/**
	 * Initialise the captcha
	 *
	 * @param	string	$id	The id of the field.
	 *
	 * @return	Boolean	True on success, false otherwise
	 *
	 * @since  2.5
	 */
	public function onInit($id)
	{
		if ($this->ayah_publisher_key == null || $this->ayah_publisher_key == '')
		{
			throw new Exception(JText::_('PLG_PLAYTHRU_ERROR_NO_PUBLISHER_KEY'));
		}

		return true;
	}

	/**
	 * Gets the challenge HTML
	 *
	 * @return  string  The HTML to be embedded in the form.
	 *
	 * @since  2.5
	 */
	public function onDisplay($name, $id, $class)
	{
		$url = 'https://' . self::AYAH_WEB_SERVICE_HOST . "/ws/script/".urlencode($this->ayah_publisher_key);

		return "<div id='AYAH'></div><script src='". $url ."' type='text/javascript' language='JavaScript'></script>";
	}

	/**
	  * Calls an HTTP POST function to verify if the user's guess was correct
	  *
	  * @return  True if the answer is correct, false otherwise
	  *
	  * @since  2.5
	  */
	public function onCheckAnswer($code)
	{
		$result = false;

		if(!$this->ayah_scoring_key)
		{
			$this->_subject->setError(JText::_('PLG_PLAYTHRU_ERROR_NO_SCORING_KEY'));
			return false;
		}

		if(!$this->session_secret)
		{
			$this->_subject->setError(JText::_('PLG_PLAYTHRU_ERROR_NO_SESSION_SECRET'));
			return false;
		}

		if ($this->session_secret)
		{
			$fields = array(
				'session_secret' => urlencode($this->session_secret),
				'scoring_key' => $this->ayah_scoring_key
				);

			$resp = $this->doHttpsPostReturnJSONArray(self::AYAH_WEB_SERVICE_HOST, "/ws/scoreGame", $fields);
			if($resp->status_code == 1)
			{
				$result = true;
			}
			else
			{
				$this->_subject->setError(JText::_('PLG_PLAYTHRU_ERR_CODE_NOT_VALID'));
			}
		}

		return $result;
	}

	/**
 	 * Do a HTTPS POST, return some JSON decoded as array (Internal function)
	 * @param $host hostname
 	 * @param $path path
	 * @param $fields associative array of fields
	 * return JSON decoded data structure or empty data structure
	 */
	protected function doHttpsPostReturnJSONArray($hostname, $path, $fields)
	{
		$result = $this->doHttpsPost($hostname, $path, $fields);

		if ($result) $result = json_decode($result);
		else
		{
			$result = array();
		}

		return $result;
	}

	protected function doHttpsPost($hostname, $path, $fields)
	{
		$result = "";
		// URLencode the post string
		$fields_string = "";
		foreach($fields as $key => $value)
		{
			if (is_array($value))
			{
				foreach ($value as $v)
				{
					$fields_string .= $key . '[]=' . $v . '&';
				}
			}
			else
			{
				$fields_string .= $key.'='.$value.'&';
			}
		}
		rtrim($fields_string,'&');

		// cURL or something else
		if (function_exists('curl_init'))
		{
			$curlsession = curl_init();
			curl_setopt($curlsession, CURLOPT_URL, "https://" . $hostname . $path);
			curl_setopt($curlsession, CURLOPT_POST, count($fields));
			curl_setopt($curlsession, CURLOPT_POSTFIELDS, $fields_string);
			curl_setopt($curlsession, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curlsession, CURLOPT_SSL_VERIFYHOST, 0);
			curl_setopt($curlsession, CURLOPT_SSL_VERIFYPEER, false);
			$result = curl_exec($curlsession);
		}
		else
		{
			// Build a header
			$http_request  = "POST $path HTTP/1.1\r\n";
			$http_request .= "Host: $hostname\r\n";
			$http_request .= "Content-Type: application/x-www-form-urlencoded;\r\n";
			$http_request .= "Content-Length: " . strlen($fields_string) . "\r\n";
			$http_request .= "User-Agent: AreYouAHuman/PHP 1.0.2\r\n";
			$http_request .= "Connection: Close\r\n";
			$http_request .= "\r\n";
			$http_request .= $fields_string ."\r\n";

			$result = '';
			$errno = $errstr = "";
			$fs = fsockopen("ssl://" . $hostname, 443, $errno, $errstr, 10);
			if( false == $fs )
			{
				die('Could not open socket');
			}
			else
			{
				fwrite($fs, $http_request);
				while (!feof($fs))
				{
					$result .= fgets($fs, 4096);
				}

				$result = explode("\r\n\r\n", $result, 2);
				$result = $result[1];
			}
		}

		return $result;
	}
}
